@@ -1,9 +1,8 @@
 import asyncio
 import logging
-import os
-import sys
+from collections import deque
 from enum import Enum
-from typing import Any, Awaitable, Callable, Dict, List, Optional, Union, cast
+from typing import Any, Awaitable, Callable, Deque, Dict, List, Optional, Union, cast
 
 from aiosignalrcore.hub.base_hub_connection import BaseHubConnection  # type: ignore
 from aiosignalrcore.hub_connection_builder import HubConnectionBuilder  # type: ignore
@@ -92,6 +91,32 @@ class OperationFetcherChannel(Enum):
     originations = 'originations'
 
 
+class CallbackExecutor:
+    def __init__(self) -> None:
+        self._queue: Deque[Awaitable] = deque()
+
+    def submit(self, fn, *args, **kwargs):
+        self._queue.append(fn(*args, **kwargs))
+
+    async def run(self):
+        while True:
+            await asyncio.sleep(0.1)
+            try:
+                coro = self._queue.popleft()
+                await coro
+            except IndexError:
+                pass
+            except asyncio.CancelledError:
+                return
+
+
+def dedup_operations(operations: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
+    return sorted(
+        list(({op['id']: op for op in operations}).values()),
+        key=lambda op: op['id'],
+    )
+
+
 class OperationFetcher:
     def __init__(
         self,
@@ -232,8 +257,7 @@ class OperationFetcher:
             while self._head <= head:
                 if self._head in self._operations:
                     operations = self._operations.pop(self._head)
-                    operations = sorted(list(({op['id']: op for op in operations}).values()), key=lambda op: op['id'])
-                    yield self._head, operations
+                    yield self._head, dedup_operations(operations)
                 self._head += 1
 
             if all(list(self._fetched.values())):
@@ -262,6 +286,7 @@ class TzktDatasource:
         self._rollback_fn: Optional[Callable[[int, int], Awaitable[None]]] = None
         self._package: Optional[str] = None
         self._proxy = TzktRequestProxy(cache)
+        self._callback_executor = CallbackExecutor()
 
     async def add_index(self, index_name: str, index_config: Union[OperationIndexConfig, BigMapIndexConfig, BlockIndexConfig]):
         self._logger.info('Adding index `%s`', index_name)
@@ -297,10 +322,17 @@ class TzktDatasource:
                     }
                 )
             ).build()
+
+            async def operation_callback(*args, **kwargs) -> None:
+                self._callback_executor.submit(self.on_operation_message, *args, **kwargs)
+
+            async def big_map_callback(*args, **kwargs) -> None:
+                self._callback_executor.submit(self.on_big_map_message, *args, **kwargs)
+
             self._client.on_open(self.on_connect)
             self._client.on_error(self.on_error)
-            self._client.on('operations', self.on_operation_message)
-            self._client.on('bigmaps', self.on_big_map_message)
+            self._client.on('operations', operation_callback)
+            self._client.on('bigmaps', big_map_callback)
 
         return self._client
 
@@ -343,7 +375,10 @@ class TzktDatasource:
 
         if not rest_only:
             self._logger.info('Starting websocket client')
-            await self._get_client().start()
+            await asyncio.gather(
+                self._get_client().start(),
+                self._callback_executor.run(),
+            )
 
     async def stop(self):
         ...
