@@ -10,6 +10,7 @@ from tortoise.exceptions import OperationalError
 from tortoise.utils import get_schema_sql
 
 import dipdup.codegen as codegen
+import dipdup.utils as utils
 from dipdup import __version__
 from dipdup.config import (
     ROLLBACK_HANDLER,
@@ -25,7 +26,6 @@ from dipdup.datasources.tzkt.datasource import TzktDatasource
 from dipdup.exceptions import ConfigurationError
 from dipdup.hasura import configure_hasura
 from dipdup.models import IndexType, State
-from dipdup.utils import reindex, tortoise_wrapper
 
 
 class DipDup:
@@ -41,7 +41,7 @@ class DipDup:
         await codegen.generate_handlers(self._config)
         await codegen.cleanup(self._config)
 
-    async def run(self) -> None:
+    async def run(self, reindex: bool) -> None:
         url = self._config.database.connection_string
         cache = isinstance(self._config.database, SqliteDatabaseConfig)
         models = f'{self._config.package}.models'
@@ -51,8 +51,8 @@ class DipDup:
         except ModuleNotFoundError as e:
             raise ConfigurationError(f'Package `{self._config.package}` not found. Have you forgot to call `init`?') from e
 
-        async with tortoise_wrapper(url, models):
-            await self.initialize_database()
+        async with utils.tortoise_wrapper(url, models):
+            await self.initialize_database(reindex)
 
             await self._config.initialize()
 
@@ -120,7 +120,7 @@ class DipDup:
 
             await asyncio.gather(*run_tasks)
 
-    async def initialize_database(self) -> None:
+    async def initialize_database(self, reindex: bool = False) -> None:
         self._logger.info('Initializing database')
 
         if isinstance(self._config.database, PostgresDatabaseConfig) and self._config.database.schema_name:
@@ -134,6 +134,10 @@ class DipDup:
         processed_schema_sql = '\n'.join(sorted(schema_sql.replace(',', '').split('\n'))).encode()
         schema_hash = hashlib.sha256(processed_schema_sql).hexdigest()
 
+        if reindex:
+            self._logger.warning('Started with `--reindex` argument, reindexing')
+            await utils.reindex()
+
         try:
             schema_state = await State.get_or_none(index_type=IndexType.schema, index_name=connection_name)
         except OperationalError:
@@ -145,4 +149,4 @@ class DipDup:
             await schema_state.save()
         elif schema_state.hash != schema_hash:
             self._logger.warning('Schema hash mismatch, reindexing')
-            await reindex()
+            await utils.reindex()
